package com.yourappname;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.telephony.SmsMessage;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import com.facebook.react.ReactActivity;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.yourappname.SmsModule; // Замените "yourappname" на имя вашего пакета

public class MainActivity extends ReactActivity {

    private BroadcastReceiver msgReceiver;
    public static final String SMS_RECEIVED_ACTION = "android.provider.Telephony.SMS_RECEIVED";
    private static final int PERMISSION_REQUEST_RECEIVE_SMS = 1;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        registerSmsReceiver();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        unregisterSmsReceiver();
    }

    private void registerSmsReceiver() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            requestSmsPermission();
        } else {
            doRegisterSmsReceiver();
        }
    }

    private void requestSmsPermission() {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.RECEIVE_SMS) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.RECEIVE_SMS}, PERMISSION_REQUEST_RECEIVE_SMS);
        } else {
            doRegisterSmsReceiver();
        }
    }

    private void doRegisterSmsReceiver() {
        msgReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                String message = getMessageFromMessageIntent(intent);
                sendEventToReactNative("received_sms", message);
            }
        };
        registerReceiver(msgReceiver, new IntentFilter(SMS_RECEIVED_ACTION));

        // Добавьте вызовы SmsModule для проверки и установки приложения по умолчанию
        checkAndSetDefaultSmsApp();
    }

    private void checkAndSetDefaultSmsApp() {
        SmsModule smsModule = new SmsModule();
        if (!smsModule.isDefaultSmsApp()) {
            smsModule.requestSetDefaultSmsApp();
        }
    }

    private void unregisterSmsReceiver() {
        if (msgReceiver != null) {
            unregisterReceiver(msgReceiver);
            msgReceiver = null;
        }
    }

    private void sendEventToReactNative(String eventName, String data) {
        getReactInstanceManager().getCurrentReactContext()
                .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                .emit(eventName, data);
    }

    private String getMessageFromMessageIntent(Intent intent) {
        // Получение данных из SMS
        StringBuilder messageBody = new StringBuilder();
        Bundle bundle = intent.getExtras();

        if (bundle != null) {
            Object[] pdusObj = (Object[]) bundle.get("pdus");
            if (pdusObj != null) {
                for (Object aPdusObj : pdusObj) {
                    SmsMessage currentMessage = SmsMessage.createFromPdu((byte[]) aPdusObj);
                    messageBody.append(currentMessage.getDisplayMessageBody());
                }
            }
        }

        return messageBody.toString();
    }
}
